/* evtgen main.
   Copyright (C) 2003 Julien Olivain and LSV, CNRS UMR 8643 & ENS Cachan.

   This file is part of evtgen.

   evtgen is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   evtgen is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with evtgen; see the file COPYING.  If not, write to
   the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * evtgen: an event generator
 **/

#include <stdlib.h>
#include <stdio.h>

#include <time.h>
#include <string.h>

#include "evtgen.h"

static void parse_cmdline(evtgen_t *ctx, int argc, char *argv[]);
static void evtgen_usage(char *prg);

int
main(int argc, char *argv[])
{
  evtgen_t *ctx;

  ctx = new_evtgen_context();
  parse_cmdline(ctx, argc, argv);
  compile_config_file(ctx);

  if (ctx->flags & MODE_VERBOSE)
    {
      fprintf_randvar_list(stderr, ctx);
      fprintf(stderr, "Event list:\n");
      fprintf_value_list(stderr, ctx, NULL, ctx->prob_tree_root, 1.0f, "", 0);
    }

  srand(ctx->prng_seed);
  ctx->cur_time = time(NULL);
  for (ctx->seq = 0; ctx->seq < ctx->max_event; ctx->seq++)
    {
      generate_string(ctx, ctx->prob_tree_root, ctx->buffer, BUFFERSZ);
      printf("%s\n", ctx->buffer);
      if (ctx->flags & MODE_REALTIME)
        {
          int var;
          var = ((frand() - 0.5f) * (ctx->bound * 2.0f));
          usleep((ctx->delay + var) * 1000);
          ctx->cur_time = time(NULL);
        }
      else
        {
          ctx->cur_time++;
        }
    }

  return (0);
}

static void
parse_cmdline(evtgen_t *ctx, int argc, char *argv[])
{
  char opt;

  while ((opt = getopt(argc, argv, "hc:s:rd:b:e:v")) != -1)
    {
      switch (opt)
        {
        case 'h':
          evtgen_usage(argv[0]);
          exit(EXIT_FAILURE);
          break;

        case 'c':
          ctx->config_file = strdup(optarg);
          break;

        case 's':
          ctx->prng_seed = atoi(optarg);
          if (ctx->prng_seed == -1)
            ctx->prng_seed = time(NULL);
          break;

        case 'r':
          ctx->flags |= MODE_REALTIME;
          break;

        case 'd':
          ctx->delay = atoi(optarg);
          break;

        case 'b':
          ctx->bound = atoi(optarg);
          if (ctx->bound > ctx->delay)
            {
              DPRINTF( ("warning: bound can't be greater than delay\n") );
              ctx->bound = ctx->delay;
            }
          break;

        case 'e':
          ctx->max_event = atoi(optarg);
          break;

        case 'v':
          ctx->flags |= MODE_VERBOSE;
          break;

        default:
          evtgen_usage(argv[0]);
          exit(EXIT_FAILURE);
        }
    }
}

static void
evtgen_usage(char *prg)
{
  fprintf(stderr, "evtgen usage :\n");
  fprintf(stderr, "%s <options>\n", prg);
  fprintf(stderr, "  -c <config_file>   configuration file (def. '%s')\n", DEFAULT_CONFIG_FILE);
  fprintf(stderr, "  -s <seed>          set the PRNG seed  (def. 0)\n");
  fprintf(stderr, "  -r                 real-time mode     (def. off)\n");
  fprintf(stderr, "  -d <delay_ms>      delay in ms        (def. %i)\n", DEFAULT_RTDELAY);
  fprintf(stderr, "  -b <bound_ms>      random var. bound  (def. %i)\n", DEFAULT_RTBOUND);
  fprintf(stderr, "  -e <events>        event number       (def. %i)\n", DEFAULT_MAX_EVENT);
  fprintf(stderr, "  -v                 verbose            (def. off)\n");
}
